@@ -75,16 +75,29 @@ void cc_qsig_op_ecma_isdn_namepres(struct cc_qsig_invokedata *invoke, struct cap
 		
 	switch (invoke->type) {
 		case 0:	/* Calling Name */
-			i->owner->cid.cid_name = strdup(callername);	/* Save name to callerid */
+#ifdef CC_AST_HAS_VERSION_1_8
+			/* ast_set_callerid updates CDR, but __ast_pbx_run updates CDR too.
+					__ast_pbx_run does not uses the channel lock and this results in destruction
+					of CDR list
+					Do notcall this function until problem resolved
+					ast_set_callerid(i->owner, NULL, callername, NULL);
+					Use code from ast_set_callerid but do not update CDR
+					*/
+				i->owner->caller.id.name.valid = 1;
+				ast_free(i->owner->caller.id.name.str);
+				i->owner->caller.id.name.str = ast_strdup(callername);
+#else
+			i->owner->cid.cid_name = ast_strdup(callername);	/* Save name to callerid */
+#endif
 			break;
 		case 1:	/* Called Name */
 		case 2: /* Connected Name */
 		case 3: /* Busy Name */
  			if (i->qsig_data.dnameid) {	/* this facility may come more than once - if so, then update this value */
 				cc_qsig_verbose( 1, VERBOSE_PREFIX_4 "  * deleting previously received name.\n", nametype, namelength, callername); 
-			 	free(i->qsig_data.dnameid);
+			 	ast_free(i->qsig_data.dnameid);
 			}
-			i->qsig_data.dnameid = strdup(callername);	/* save name as destination in qsig specific fields */
+			i->qsig_data.dnameid = ast_strdup(callername);	/* save name as destination in qsig specific fields */
 									/* there's no similarly field in asterisk */
 			break;
 		default:
@@ -112,7 +125,7 @@ void cc_qsig_op_ecma_isdn_namepres(struct cc_qsig_invokedata *invoke, struct cap
  * returns
  * 	always 0
  */
-int cc_qsig_encode_ecma_name_invoke(unsigned char * buf, unsigned int *idx, struct cc_qsig_invokedata *invoke, struct capi_pvt *i, int nametype, char * name)
+int cc_qsig_encode_ecma_name_invoke(unsigned char * buf, unsigned int *idx, struct cc_qsig_invokedata *invoke, struct capi_pvt *i, int nametype, const char * name)
 {
 	unsigned char namebuf[51];
 	unsigned char data[255];
@@ -382,40 +395,47 @@ void cc_qsig_encode_ecma_calltransfer(unsigned char * buf, unsigned int *idx, st
 
 	if (param) { /* got Call Transfer Parameters */
 		if (info) {
-			cid = strsep(&param, "|");
+			cid = strsep(&param, COMMANDSEPARATOR);
 			cidlen = strlen(cid);
 			if (cidlen > 20)	/* HACK: stop action here, maybe we have invalid data */
 				cidlen = 20;
 		} else {
-			char *tmp = strsep(&param, "|");
+			char *tmp = strsep(&param, COMMANDSEPARATOR);
 			tmp = NULL;
-			cid = strsep(&param, "|");
+			cid = strsep(&param, COMMANDSEPARATOR);
 			cidlen = strlen(cid);
 			if (cidlen > 20)	/* HACK: stop action here, maybe we have invalid data */
 				cidlen = 20;
 			
-			ccanswer = strsep(&param, "|");
+			ccanswer = strsep(&param, COMMANDSEPARATOR);
 			if (ccanswer[0])
 				icanswer = ccanswer[0] - 0x30;
 		}
 	} else {
-/* 		cid = strdup(i->owner->cid.cid_num);*/ /* Here we get the Asterisk extension */
+/* 		cid = ast_strdup(i->owner->cid.cid_num);*/ /* Here we get the Asterisk extension */
 		if (info) { /* info is >0 on outbound channel (second facility) */
  			struct capi_pvt *ii = capi_find_interface_by_plci(i->qsig_data.partner_plci);
 			
-			cid = strdup(i->cid);
+			cid = ast_strdup(i->cid);
 			cidlen = strlen(cid);
 			
 			if (ii) {
 				/* send callers name to user B */
+#ifdef CC_AST_HAS_VERSION_1_8
+				if (ii->owner->caller.id.name.valid ) {
+					name = ast_strdupa(S_COR(ii->owner->caller.id.name.valid, ii->owner->caller.id.name.str, ""));
+					namelength = strlen(name);
+				}
+#else
 				if (ii->owner->cid.cid_name) {
 					name = ast_strdupa(ii->owner->cid.cid_name);
 					namelength = strlen(name);
 				}
+#endif
 			}
 		} else { /* have to build first facility - send destination number back to inbound channel */
 			struct capi_pvt *ii = capi_find_interface_by_plci(i->qsig_data.partner_plci);
-			cid = strdup(ii->dnid);
+			cid = ast_strdup(ii->dnid);
 			cidlen = strlen(cid);
 			
 			if (ii) {
@@ -479,7 +499,7 @@ void cc_qsig_encode_ecma_calltransfer(unsigned char * buf, unsigned int *idx, st
 	cc_qsig_verbose( 0, VERBOSE_PREFIX_4 "  * Sending QSIG_CT: %i->%s\n", info, cid);
 	
 	if (cid)
-		free(cid);
+		ast_free(cid);
 	
 }
 
@@ -552,7 +572,7 @@ unsigned int cc_qsig_decode_ecma_calltransfer(struct cc_qsig_invokedata *invoke,
 		if (data[myidx] == ASN1_TC_APPLICATION) {
 			myidx++;
 			/* TODO: check size -> could be bigger than 256 bytes - MSB is set then */
-			ctc->basicCallInfoElements = malloc(data[myidx]);
+			ctc->basicCallInfoElements = ast_malloc(data[myidx]);
 			if (ctc->basicCallInfoElements) {
 				memcpy(ctc->basicCallInfoElements, &data[myidx+1], data[myidx] );
 			} else {
@@ -566,7 +586,7 @@ unsigned int cc_qsig_decode_ecma_calltransfer(struct cc_qsig_invokedata *invoke,
 		if (data[myidx] != ASN1_ENUMERATED) { /* Maybe we get an name (OPTIONAL) */
 			myidx += cc_qsig_asn197no_get_name(ct_name, ASN197NO_NAME_STRSIZE+1, &namelength, &myidx, data );
 			if (namelength)
-				ctc->redirectionName = strdup(ct_name);
+				ctc->redirectionName = ast_strdup(ct_name);
 		}
 	}
 	
@@ -607,12 +627,12 @@ void cc_qsig_encode_ecma_sscalltransfer(unsigned char * buf, unsigned int *idx,
 	char c[255];
 	int ix = 0;
 
-	cidsrc = strsep(&param, "|");
+	cidsrc = strsep(&param, COMMANDSEPARATOR);
 	srclen = strlen(cidsrc);
 	if (srclen > 20)	/* HACK: stop action here, maybe we have invalid data */
 		srclen = 20;
 	
-	ciddst = strsep(&param, "|");
+	ciddst = strsep(&param, COMMANDSEPARATOR);
 	dstlen = strlen(ciddst);
 	if (dstlen > 20)	/* HACK: stop action here, maybe we have invalid data */
 		dstlen = 20;
@@ -718,8 +738,8 @@ void cc_qsig_op_ecma_isdn_prpropose(struct cc_qsig_invokedata *invoke, struct ca
 	}
 	
 
-	i->qsig_data.pr_propose_cid  = strdup(callid);
-	i->qsig_data.pr_propose_pn = strdup(reroutingnr);
+	i->qsig_data.pr_propose_cid  = ast_strdup(callid);
+	i->qsig_data.pr_propose_pn = ast_strdup(reroutingnr);
 	
 	cc_qsig_verbose( 0, VERBOSE_PREFIX_4 "  * Got QSIG_PATHREPLACEMENT_PROPOSE Call identity: %s, Party number: %s (%i)\n", callid, reroutingnr, temp);
 	
