@@ -122,27 +122,27 @@ _cstruct capi_rtp_ncpi(struct capi_pvt *i)
 	if ((i) && (i->owner) &&
 	    (i->bproto == CC_BPROTO_RTP)) {
 		switch(i->codec) {
-		case AST_FORMAT_ALAW:
+		case CC_FORMAT_ALAW:
 			ncpi = NCPI_voice_over_ip_alaw;
 			break;
-		case AST_FORMAT_ULAW:
+		case CC_FORMAT_ULAW:
 			ncpi = NCPI_voice_over_ip_ulaw;
 			break;
-		case AST_FORMAT_GSM:
+		case CC_FORMAT_GSM:
 			ncpi = NCPI_voice_over_ip_gsm;
 			break;
-		case AST_FORMAT_G723_1:
+		case CC_FORMAT_G723_1:
 			ncpi = NCPI_voice_over_ip_g723;
 			break;
-		case AST_FORMAT_G726:
+		case CC_FORMAT_G726:
 			ncpi = NCPI_voice_over_ip_g726;
 			break;
-		case AST_FORMAT_G729A:
+		case CC_FORMAT_G729A:
 			ncpi = NCPI_voice_over_ip_g729;
 			break;
 		default:
 			cc_log(LOG_ERROR, "%s: format %s(%d) invalid.\n",
-				i->vname, ast_getformatname(i->codec), i->codec);
+				i->vname, cc_getformatname(i->codec), i->codec);
 			break;
 		}
 	}
@@ -155,19 +155,33 @@ _cstruct capi_rtp_ncpi(struct capi_pvt *i)
  */
 int capi_alloc_rtp(struct capi_pvt *i)
 {
+#ifndef CC_AST_HAS_VERSION_10_0 /* Use vocoder without RTP framing */
+#ifdef CC_AST_HAS_AST_SOCKADDR
+	struct ast_sockaddr addr;
+	struct ast_sockaddr us;
+#else
 	struct ast_hostent ahp;
 	struct hostent *hp;
 	struct in_addr addr;
 	struct sockaddr_in us;
+#endif
 #ifndef CC_AST_HAS_VERSION_1_4
 	char temp[MAXHOSTNAMELEN];
 #endif
 
+#ifdef CC_AST_HAS_AST_SOCKADDR
+	ast_sockaddr_parse(&addr, "localhost:0", 0);
+#else
 	hp = ast_gethostbyname("localhost", &ahp);
 	memcpy(&addr, hp->h_addr, sizeof(addr));
+#endif
 
 #ifdef CC_AST_HAS_RTP_ENGINE_H
+#ifdef CC_AST_HAS_AST_SOCKADDR
+	i->rtp = ast_rtp_instance_new(NULL, NULL, &addr, NULL);
+#else
 	i->rtp = ast_rtp_instance_new(NULL, NULL, (struct sockaddr_in *)&addr, NULL);
+#endif
 #else
 	i->rtp = ast_rtp_new_with_bindaddr(NULL, NULL, 0, 0, addr);
 #endif
@@ -185,14 +199,23 @@ int capi_alloc_rtp(struct capi_pvt *i)
 #endif
 	cc_verbose(2, 1, VERBOSE_PREFIX_4 "%s: alloc rtp socket on %s:%d\n",
 		i->vname,
+#ifdef CC_AST_HAS_AST_SOCKADDR
+		ast_sockaddr_stringify(&us), ntohs(ast_sockaddr_port(&us)));
+#else
 #ifdef CC_AST_HAS_VERSION_1_4
 		ast_inet_ntoa(us.sin_addr),
 #else
 		ast_inet_ntoa(temp, sizeof(temp), us.sin_addr),
 #endif
 		ntohs(us.sin_port));
+#endif
 	i->timestamp = 0;
 	return 0;
+#else
+	i->rtp = 0;
+	cc_log(LOG_ERROR, "%s: use vocoder\n", i->vname);
+	return 1;
+#endif
 }
 
 /*
@@ -200,14 +223,17 @@ int capi_alloc_rtp(struct capi_pvt *i)
  */
 int capi_write_rtp(struct capi_pvt *i, struct ast_frame *f)
 {
+#ifndef CC_AST_HAS_VERSION_10_0 /* Use vocoder */
+#ifdef CC_AST_HAS_AST_SOCKADDR
+	struct ast_sockaddr us;
+#else
 	struct sockaddr_in us;
+	socklen_t uslen = sizeof(us);
+#endif
 	int len;
-	socklen_t uslen;
 	unsigned int *rtpheader;
 	unsigned char buf[256];
 
-	uslen = sizeof(us);
-
 	if (!(i->rtp)) {
 		cc_log(LOG_ERROR, "rtp struct is NULL\n");
 		return -1;
@@ -228,12 +254,16 @@ int capi_write_rtp(struct capi_pvt *i, struct ast_frame *f)
 	}
 
 	while(1) {
+#ifdef CC_AST_HAS_AST_SOCKADDR
+		len = ast_recvfrom(ast_rtp_instance_fd(i->rtp, 0), buf, sizeof(buf), 0, &us);
+#else
 #ifdef CC_AST_HAS_RTP_ENGINE_H
 		len = recvfrom(ast_rtp_instance_fd(i->rtp, 0),
 			buf, sizeof(buf), 0, (struct sockaddr *)&us, &uslen);
 #else
 		len = recvfrom(ast_rtp_fd(i->rtp),
 			buf, sizeof(buf), 0, (struct sockaddr *)&us, &uslen);
+#endif
 #endif
 		if (len <= 0)
 			break;
@@ -260,7 +290,7 @@ int capi_write_rtp(struct capi_pvt *i, struct ast_frame *f)
 		i->send_buffer_handle++;
 
 		cc_verbose(6, 1, VERBOSE_PREFIX_4 "%s: RTP write for NCCI=%#x len=%d(%d) %s ts=%x\n",
-			i->vname, i->NCCI, len, f->datalen, ast_getformatname(FRAME_SUBCLASS_CODEC(f->subclass)),
+			i->vname, i->NCCI, len, f->datalen, cc_getformatname(GET_FRAME_SUBCLASS_CODEC(f->subclass)),
 			i->timestamp);
 
 		capi_sendf(NULL, 0, CAPI_DATA_B3_REQ, i->NCCI, get_capi_MessageNumber(),
@@ -272,6 +302,8 @@ int capi_write_rtp(struct capi_pvt *i, struct ast_frame *f)
 		);
 	}
 
+#endif
+
 	return 0;
 }
 
@@ -280,8 +312,13 @@ int capi_write_rtp(struct capi_pvt *i, struct ast_frame *f)
  */
 struct ast_frame *capi_read_rtp(struct capi_pvt *i, unsigned char *buf, int len)
 {
+#ifndef CC_AST_HAS_VERSION_10_0 /* Use vocoder */
 	struct ast_frame *f;
+#ifdef CC_AST_HAS_AST_SOCKADDR
+	struct ast_sockaddr us;
+#else
 	struct sockaddr_in us;
+#endif
 
 	if (!(i->owner))
 		return NULL;
@@ -299,10 +336,14 @@ struct ast_frame *capi_read_rtp(struct capi_pvt *i, unsigned char *buf, int len)
 	ast_rtp_set_peer(i->rtp, &us);
 #endif
 
+#ifdef CC_AST_HAS_AST_SOCKADDR
+	if (len != ast_sendto(ast_rtp_instance_fd(i->rtp, 0), buf, len, 0, &us))
+#else
 #ifdef CC_AST_HAS_RTP_ENGINE_H
 	if (len != sendto(ast_rtp_instance_fd(i->rtp, 0), buf, len, 0, (struct sockaddr *)&us, sizeof(us)))
 #else
 	if (len != sendto(ast_rtp_fd(i->rtp), buf, len, 0, (struct sockaddr *)&us, sizeof(us)))
+#endif
 #endif
 	{
 		cc_verbose(4, 1, VERBOSE_PREFIX_3 "%s: RTP sendto error\n",
@@ -321,17 +362,20 @@ struct ast_frame *capi_read_rtp(struct capi_pvt *i, unsigned char *buf, int len)
 			return NULL;
 		}
 		cc_verbose(6, 1, VERBOSE_PREFIX_4 "%s: DATA_B3_IND RTP NCCI=%#x len=%d %s (read/write=%d/%d)\n",
-			i->vname, i->NCCI, len, ast_getformatname(FRAME_SUBCLASS_CODEC(f->subclass)),
+			i->vname, i->NCCI, len, cc_getformatname(GET_FRAME_SUBCLASS_CODEC(f->subclass)),
 			i->owner->readformat, i->owner->writeformat);
-		if (i->owner->nativeformats != FRAME_SUBCLASS_CODEC(f->subclass)) {
+		if (i->owner->nativeformats != GET_FRAME_SUBCLASS_CODEC(f->subclass)) {
 			cc_verbose(3, 1, VERBOSE_PREFIX_3 "%s: DATA_B3_IND RTP nativeformats=%d, but subclass=%ld\n",
-				i->vname, i->owner->nativeformats, FRAME_SUBCLASS_CODEC(f->subclass));
-			i->owner->nativeformats = FRAME_SUBCLASS_CODEC(f->subclass);
+				i->vname, i->owner->nativeformats, GET_FRAME_SUBCLASS_CODEC(f->subclass));
+			i->owner->nativeformats = GET_FRAME_SUBCLASS_CODEC(f->subclass);
 			ast_set_read_format(i->owner, i->owner->readformat);
 			ast_set_write_format(i->owner, i->owner->writeformat);
 		}
 	}
 	return f;
+#else
+	return NULL;
+#endif
 }
 
 /*
@@ -404,33 +448,64 @@ void voice_over_ip_profile(struct cc_capi_controller *cp)
 
 	cc_verbose(3, 0, VERBOSE_PREFIX_4 "RTP codec: ");
 	if (payload1 & 0x00000100) {
-		cp->rtpcodec |= AST_FORMAT_ALAW;
+		cp->rtpcodec |= CC_FORMAT_ALAW;
 		cc_verbose(3, 0, "G.711-alaw ");
 	}
 	if (payload1 & 0x00000001) {
-		cp->rtpcodec |= AST_FORMAT_ULAW;
+		cp->rtpcodec |= CC_FORMAT_ULAW;
 		cc_verbose(3, 0, "G.711-ulaw ");
 	}
 	if (payload1 & 0x00000008) {
-		cp->rtpcodec |= AST_FORMAT_GSM;
+		cp->rtpcodec |= CC_FORMAT_GSM;
 		cc_verbose(3, 0, "GSM ");
 	}
 	if (payload1 & 0x00000010) {
-		cp->rtpcodec |= AST_FORMAT_G723_1;
+		cp->rtpcodec |= CC_FORMAT_G723_1;
 		cc_verbose(3, 0, "G.723.1 ");
 	}
 	if (payload1 & 0x00000004) {
-		cp->rtpcodec |= AST_FORMAT_G726;
+		cp->rtpcodec |= CC_FORMAT_G726;
 		cc_verbose(3, 0, "G.726 ");
 	}
 	if (payload1 & 0x00040000) {
-		cp->rtpcodec |= AST_FORMAT_G729A;
-		cc_verbose(3, 0, "G.729");
+		cp->rtpcodec |= CC_FORMAT_G729A;
+		cc_verbose(3, 0, "G.729 ");
 	}
 	if (payload1 & (1U << 27)) {
-		cp->rtpcodec |= AST_FORMAT_ILBC;
-		cc_verbose(3, 0, "iLBC");
+		cp->rtpcodec |= CC_FORMAT_ILBC;
+		cc_verbose(3, 0, "iLBC ");
 	}
+#ifdef CC_FORMAT_G722
+	if (payload1 & (1U << 9)) {
+		cp->rtpcodec |= CC_FORMAT_G722;
+		cc_verbose(3, 0, "G.722 ");
+	}
+#endif
+#if defined(CC_FORMAT_SIREN7) && defined(CC_FORMAT_SIREN14)
+	if (payload1 & (1U << 24)) {
+#ifdef CC_FORMAT_SIREN7
+		cp->rtpcodec |= CC_FORMAT_SIREN7;
+		cc_verbose(3, 0, "siren7 ");
+#endif
+#ifdef CC_FORMAT_SIREN14
+		cp->rtpcodec |= CC_FORMAT_SIREN14;
+		cc_verbose(3, 0, "siren14 ");
+#endif
+	}
+#endif
+#if defined(CC_FORMAT_SLINEAR) || defined(CC_FORMAT_SLINEAR16)
+	if (payload1 & (1U << 1)) {
+#if defined(CC_FORMAT_SLINEAR)
+		cp->rtpcodec |= CC_FORMAT_SLINEAR;
+		cc_verbose(3, 0, "slin ");
+#endif
+#if defined(CC_FORMAT_SLINEAR16)
+		cp->rtpcodec |= CC_FORMAT_SLINEAR16;
+		cc_verbose(3, 0, "slin16 ");
+#endif
+	}
+#endif
+
 	cc_verbose(3, 0, "\n");
 }
 
