@@ -18,6 +18,9 @@
 #include "chan_capi.h"
 #include "chan_capi_supplementary.h"
 #include "chan_capi_utils.h"
+#if defined(CC_AST_HAS_EVENT_MWI)
+#include <asterisk/event.h>
+#endif
 
 
 #define CCBSNR_TYPE_CCBS 1
@@ -65,7 +68,7 @@ static void del_old_ccbsnr(void)
 			} else {
 				tmp->next = ccbsnr->next;
 			}
-			free(ccbsnr);
+			ast_free(ccbsnr);
 			break;
 		}
 		tmp = ccbsnr;
@@ -86,7 +89,7 @@ void cleanup_ccbsnr(void)
 	while (ccbsnr) {
 		tmp = ccbsnr;
 		ccbsnr = ccbsnr->next;
-		free(tmp);
+		ast_free(tmp);
 	}
 	cc_mutex_unlock(&ccbsnr_lock);
 }
@@ -122,7 +125,7 @@ static void new_ccbsnr_id(char type, unsigned int plci,
 	char buffer[CAPI_MAX_STRING];
 	struct ccbsnr_s *ccbsnr;
 
-	ccbsnr = malloc(sizeof(struct ccbsnr_s));
+	ccbsnr = ast_malloc(sizeof(struct ccbsnr_s));
 	if (ccbsnr == NULL) {
 		cc_log(LOG_ERROR, "Unable to allocate CCBS/CCNR struct.\n");
 		return;
@@ -259,7 +262,7 @@ static void del_ccbsnr_ref(unsigned int plci, _cword ref)
 			} else {
 				tmp->next = ccbsnr->next;
 			}
-			free(ccbsnr);
+			ast_free(ccbsnr);
 			cc_verbose(1, 1, VERBOSE_PREFIX_3 CC_MESSAGE_NAME
 				": PLCI=%#x CCBS/CCNR removed ref=0x%04x\n", plci, ref);
 			break;
@@ -319,7 +322,7 @@ static void del_ccbsnr_id(unsigned int plci, _cword id)
 				} else {
 					tmp->next = ccbsnr->next;
 				}
-				free(ccbsnr);
+				ast_free(ccbsnr);
 				cc_verbose(1, 1, VERBOSE_PREFIX_3 CC_MESSAGE_NAME ": PLCI=%#x CCBS/CCNR removed "
 					"id=0x%04x state=%d\n",	plci, id, oldstate);
 			} else {
@@ -399,14 +402,24 @@ static void	ccbsnr_remote_user_free(_cmsg *CMSG, char type, unsigned int PLCI, _
 
 	c->priority = ccbsnr->priority;
 
+#ifdef CC_AST_HAS_VERSION_1_8
+  /*! \todo verify if necessary/complete */
+	c->connected.id.number.valid = 1;
+	ast_free (c->connected.id.number.str);
+	c->connected.id.number.str = ast_strdup(handlename);
+
+	ast_free (c->dialed.number.str);
+	c->dialed.number.str = ast_strdup (ccbsnr->exten);
+#else
 	if (c->cid.cid_num) {
-		free(c->cid.cid_num);
+		ast_free(c->cid.cid_num);
 	}
-	c->cid.cid_num = strdup(handlename);
+	c->cid.cid_num = ast_strdup(handlename);
 	if (c->cid.cid_dnid) {
-		free(c->cid.cid_dnid);
+		ast_free(c->cid.cid_dnid);
 	}
-	c->cid.cid_dnid = strdup(ccbsnr->exten);
+	c->cid.cid_dnid = ast_strdup(ccbsnr->exten);
+#endif
 
 #ifndef CC_AST_HAS_EXT2_CHAN_ALLOC
 	cc_copy_string(c->context, ccbsnr->context, sizeof(c->context));
@@ -561,6 +574,87 @@ int handle_facility_indication_supplementary(
 		cc_verbose(1, 1, VERBOSE_PREFIX_3 "contr%d: PLCI=%#x CCBS B-free ref=0x%04x\n",
 			PLCI & 0xff, PLCI, infoword);
 		break;
+	case 0x8014: {/* MWI indication, stateless */
+		const unsigned char* info = &FACILITY_IND_FACILITYINDICATIONPARAMETER(CMSG)[4];
+		unsigned short basicService     = read_capi_word(info);  info += 2;
+		unsigned int   numberOfMessages = read_capi_dword(info); info += 4;
+		unsigned short messageStatus    = read_capi_word(info);  info += 2;
+		unsigned short messageReference = read_capi_word(info);  info += 2;
+		char controllingUserNumberName[AST_MAX_EXTENSION];
+		char controllingUserProvidedNumberName[AST_MAX_EXTENSION];
+		char mwiTimeName[AST_MAX_EXTENSION];
+		char mailboxName[AST_MAX_EXTENSION];
+
+		if (info[0] > 3) {
+			memcpy(controllingUserNumberName,
+						&info[4],
+						MIN(AST_MAX_EXTENSION-1, info[0] - 3));
+			controllingUserNumberName[MIN(AST_MAX_EXTENSION-1, info[0] - 3)] = 0;
+		} else {
+			controllingUserNumberName[0] = 0;
+		}
+		info += info[0] + 1;
+
+		if (info[0] > 3) {
+			memcpy(controllingUserProvidedNumberName,
+						&info[4],
+						MIN(AST_MAX_EXTENSION-1, info[0] - 3));
+			controllingUserProvidedNumberName[MIN(AST_MAX_EXTENSION-1, info[0] - 3)] = 0;
+		} else {
+			controllingUserProvidedNumberName[0] = 0;
+		}
+		info += info[0] + 1;
+
+		if (info[0] != 0) {
+			memcpy(mwiTimeName, &info[1], MIN(AST_MAX_EXTENSION-1, info[0]));
+			mwiTimeName[MIN(AST_MAX_EXTENSION-1, info[0])] = 0;
+		} else {
+			mwiTimeName[0] = 0;
+		}
+		info += info[0] + 1;
+
+		if (info[0] > 1) {
+			memcpy(mailboxName, &info[2], MIN(AST_MAX_EXTENSION-1, info[0]));
+			mailboxName[MIN(AST_MAX_EXTENSION-1, info[0]-1)] = 0;
+		} else {
+			mailboxName[0] = 0;
+		}
+
+		if (messageStatus == 0 || messageStatus == 1) {
+			cc_verbose(4, 0, VERBOSE_PREFIX_4 "CAPI%u Rx MWI %s for '%s@CAPI_Remote %s %s time '%s' %d messages ref %d service %d\n",
+								PLCI & 0xff,
+								messageStatus == 0 ? "add" : "del", mailboxName, controllingUserNumberName, controllingUserProvidedNumberName,
+                mwiTimeName, numberOfMessages, messageReference, basicService);
+			if (messageStatus == 0 && mailboxName[0] != 0) {
+#if defined(CC_AST_HAS_EVENT_MWI)
+				struct ast_event *event;
+
+				if ((event = ast_event_new(AST_EVENT_MWI,
+																		AST_EVENT_IE_MAILBOX, AST_EVENT_IE_PLTYPE_STR, mailboxName,
+																		AST_EVENT_IE_CONTEXT, AST_EVENT_IE_PLTYPE_STR, "CAPI_Remote",
+																		AST_EVENT_IE_NEWMSGS, AST_EVENT_IE_PLTYPE_UINT, MAX(1,numberOfMessages),
+																		AST_EVENT_IE_OLDMSGS, AST_EVENT_IE_PLTYPE_UINT, 0,
+																		AST_EVENT_IE_END))) {
+					ast_event_queue_and_cache(event);
+				}
+#endif
+			}
+		} else {
+			cc_verbose(4, 0, VERBOSE_PREFIX_1 "CAPI%u Rx MWI %s for '%s@default %s %s time '%s' service %d\n",
+								messageStatus == 0 ? "add" : "del", mailboxName, controllingUserNumberName, controllingUserProvidedNumberName,
+								mwiTimeName, basicService);
+		}
+	} return ret;
+	case 0x8000: /* Hold notification */
+		if ((i->isdnstate & CAPI_ISDN_STATE_EC) != 0) {
+			cc_verbose(4, 0, VERBOSE_PREFIX_1 "%s: EC reset\n", i->vname);
+			capi_echo_canceller(i, EC_FUNCTION_DISABLE);
+			capi_echo_canceller(i, EC_FUNCTION_ENABLE);
+		}
+		return ret;
+
+	default:
+		break;
 	}
 
 	if (!i) {
@@ -618,6 +712,11 @@ int handle_facility_indication_supplementary(
 			i->vname, PLCI, infoword);
 		show_capi_info(i, infoword);
 		break;
+	case 0x000d: /* CD */
+		cc_verbose(1, 1, VERBOSE_PREFIX_3 "%s: PLCI=%#x CD Service Reason=0x%04x\n",
+			i->vname, PLCI, infoword);
+		show_capi_info(i, infoword);
+		break;
 	case 0x8013: /* CCBS info retain */
 		cc_verbose(1, 1, VERBOSE_PREFIX_3 "%s: PLCI=%#x CCBS unique id=0x%04x\n",
 			i->vname, PLCI, infoword);
@@ -649,7 +748,7 @@ int handle_facility_indication_supplementary(
  * CAPI FACILITY_CONF supplementary
  */
 void handle_facility_confirmation_supplementary(
-	_cmsg *CMSG, unsigned int PLCI, unsigned int NCCI, struct capi_pvt **i)
+	_cmsg *CMSG, unsigned int PLCI, unsigned int NCCI, struct capi_pvt **i, struct ast_channel** interface_owner)
 {
 	_cword function;
 	_cword serviceinfo;
@@ -694,7 +793,7 @@ void handle_facility_confirmation_supplementary(
 	case 0x0012: /* CCBS call */
 		cc_verbose(2, 1, VERBOSE_PREFIX_3 "%s: CCBS call confirmation (0x%04x) (PLCI=%#x)\n",
 			name, serviceinfo, PLCI);
-		capidev_handle_connection_conf(i, PLCI, FACILITY_CONF_INFO(CMSG), HEADER_MSGNUM(CMSG));
+		capidev_handle_connection_conf(i, PLCI, FACILITY_CONF_INFO(CMSG), HEADER_MSGNUM(CMSG), interface_owner);
 		break;
 	default:
 		cc_verbose(3, 1, VERBOSE_PREFIX_3 "%s: unhandled FACILITY_CONF supplementary function %04x\n",
@@ -712,7 +811,7 @@ int pbx_capi_ccpartybusy(struct ast_channel *c, char *data)
 	struct ccbsnr_s *ccbsnr;
 	char partybusy = 0;
 
-	slinkageid = strsep(&data, "|");
+	slinkageid = strsep(&data, COMMANDSEPARATOR);
 	yesno = data;
 	
 	if (slinkageid) {
@@ -806,9 +905,9 @@ int pbx_capi_ccbs(struct ast_channel *c, char *data)
 	MESSAGE_EXCHANGE_ERROR error;
 	unsigned int ccbsnrstate;
 
-	slinkageid = strsep(&data, "|");
-	context = strsep(&data, "|");
-	exten = strsep(&data, "|");
+	slinkageid = strsep(&data, COMMANDSEPARATOR);
+	context = strsep(&data, COMMANDSEPARATOR);
+	exten = strsep(&data, COMMANDSEPARATOR);
 	priority = data;
 
 	if (slinkageid) {
